;    ____
;   //_//\  S I M P L E 
;  /   / /  L I N K - for use with the
; /___/ /   TI-83 Plus calculator.
; \_o_\/
;   |
;   v
;
; These routines aim to make your multiplayer-calc game making a LOT easier. These routines
; are easy to set up and use... much more than any others I've seen. They take care of:
; -Automatic synching of data! (no need to specify 'calc#1, calc#2' in a game!)
; -Error handling- no more 'ERR:LINK' messages! Customise 'em as you like 'em.
;
; These routines are best for swapping arrays. Picture them as an 'LDIR' command - except
; with these you are copying your data to the remote calculator (and he is copying his to
; yours) all with one simple command!
;
; Setting up:
; First, copy the next two lines into the top of your program:
;
; #define On_Error ld hl,Custom_Err \ call APP_PUSH_ERRORH
; #define Off_Error call APP_POP_ERRORH
;
; These are extremely important! Now, in your program, add the line at the top:
; On_Error
; This sets up the calc for the custom error handler.
; At the end of your code, add the line:
; Off_Error
; This, unsurprisingly, switches off the custom error handler.
; NOTE THAT IF AN ERROR HAPPENS, you'll need to reinstall the error handler with an 'On_Error'
; afterwards - ERR:LINKS are deadly!
;
; Now you've done that, all you need to know are the commands.
;
; The only one you need to touch is 'Swap_Vars'. It works like this:
;
; IN=> hl ... 'Your' data, to be sent.
;      de ... 'Their' data, where would you like to store it?
;       a ... How many bytes to swap.
;
; OUT=> Data is swapped.
;       b = 0 ... Link failed
;       b = 1 ... Link succeeded
;
; B will only ever be a '0' if the GRAPH key is held during the game. This is the emergency
; 'AAARGH!' button, if the link completely locks up.

#define SEND_BYTE bcall(_sendabyte)
#define REC_BYTE bcall(_recabyteio)
#define Wait_Link_Send call Wait_Link


;--------==[ CUSTOM ERROR HANDLER ]==--------------


Custom_Err:
	bcall(_clrlcdfull)
	bcall(_homeup)
	ld a,3
	ld (currow),a
	ld hl,Link_Fail	
	bcall(_puts)
Key_Lp_Err:
	bcall(_getcsc)
	cp sk2nd
	jr nz,Key_Lp_Err
	jp Start

Link_Fail:
	.db "* Link Failure *",0

;-----------------------------------------------------



Swap_Vars:
	di
	ld (Var_Progress_To),hl
	ld (Var_Progress_From),de
	ld (Num_To_Swap),a
	ld (Swapped_Num),a

Swap_To_Loop:
	ld hl,(Var_Progress_To)
	ld a,(hl)
	inc hl
	ld (Var_Progress_To),hl
	SEND_BYTE 
	ld a,(Swapped_Num)
	dec a
	ld (Swapped_Num),a
	or a
	jr nz,Swap_To_Loop


	ld a,(Num_To_Swap)
	ld (Swapped_Num),a

Swap_From_Loop:
	Wait_Link_Send

	ld a,b
	or a
	ret z
	REC_BYTE

	ld hl,(Var_Progress_From)
	ld (hl),a
	inc hl
	ld (Var_Progress_From),hl
	
	ld a,(Swapped_Num)
	dec a
	ld (Swapped_Num),a
	or a
	jr nz,Swap_From_Loop
	
	ld a,d0hd1h
	out (BPORT),a		;Leave port 'receiveable'


	ld b,1
	ret

Wait_Link:
	ld b,0
	ld a,$bf
	out (1),a
	in a,(1)
	cp 254
	jr nz,NotQuit
	Off_Error
	ret
NotQuit:
	ld b,1
	nop
	in a,(BPORT)
	and 3
	cp 3
	jr z,Wait_Link

	ld a,d0hd1h
	out (BPORT),a		;Leave port 'receiveable'
	ret

Var_Progress_To:
	.dw 0
Var_Progress_From:
	.dw 0

Num_To_Swap:
	.db 0
Swapped_Num:
	.db 0


